# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

import pytest

# Import Salt Libs
import salt.modules.systemd_service as systemd
import salt.utils.systemd
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing Libs
from tests.support.unit import TestCase

_SYSTEMCTL_STATUS = {
    "sshd.service": {
        "stdout": """\
* sshd.service - OpenSSH Daemon
   Loaded: loaded (/usr/lib/systemd/system/sshd.service; disabled; vendor preset: disabled)
   Active: inactive (dead)""",
        "stderr": "",
        "retcode": 3,
        "pid": 12345,
    },
    "foo.service": {
        "stdout": """\
* foo.service
   Loaded: not-found (Reason: No such file or directory)
   Active: inactive (dead)""",
        "stderr": "",
        "retcode": 3,
        "pid": 12345,
    },
}

# This reflects systemd >= 231 behavior
_SYSTEMCTL_STATUS_GTE_231 = {
    "bar.service": {
        "stdout": "Unit bar.service could not be found.",
        "stderr": "",
        "retcode": 4,
        "pid": 12345,
    },
}

_LIST_UNIT_FILES = """\
service1.service                           enabled
service2.service                           disabled
service3.service                           static
timer1.timer                               enabled
timer2.timer                               disabled
timer3.timer                               static"""


class SystemdTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.modules.systemd
    """

    def setup_loader_modules(self):
        return {systemd: {}}

    def test_systemctl_reload(self):
        """
            Test to Reloads systemctl
        """
        mock = MagicMock(
            side_effect=[
                {"stdout": "Who knows why?", "stderr": "", "retcode": 1, "pid": 12345},
                {"stdout": "", "stderr": "", "retcode": 0, "pid": 54321},
            ]
        )
        with patch.dict(systemd.__salt__, {"cmd.run_all": mock}):
            self.assertRaisesRegex(
                CommandExecutionError,
                "Problem performing systemctl daemon-reload: Who knows why?",
                systemd.systemctl_reload,
            )
            self.assertTrue(systemd.systemctl_reload())

    def test_get_enabled(self):
        """
        Test to return a list of all enabled services
        """
        cmd_mock = MagicMock(return_value=_LIST_UNIT_FILES)
        listdir_mock = MagicMock(return_value=["foo", "bar", "baz", "README"])
        sd_mock = MagicMock(
            return_value=set([x.replace(".service", "") for x in _SYSTEMCTL_STATUS])
        )
        access_mock = MagicMock(
            side_effect=lambda x, y: x
            != os.path.join(systemd.INITSCRIPT_PATH, "README")
        )
        sysv_enabled_mock = MagicMock(side_effect=lambda x, _: x == "baz")

        with patch.dict(systemd.__salt__, {"cmd.run": cmd_mock}):
            with patch.object(os, "listdir", listdir_mock):
                with patch.object(systemd, "_get_systemd_services", sd_mock):
                    with patch.object(os, "access", side_effect=access_mock):
                        with patch.object(systemd, "_sysv_enabled", sysv_enabled_mock):
                            self.assertListEqual(
                                systemd.get_enabled(),
                                ["baz", "service1", "timer1.timer"],
                            )

    def test_get_disabled(self):
        """
        Test to return a list of all disabled services
        """
        cmd_mock = MagicMock(return_value=_LIST_UNIT_FILES)
        # 'foo' should collide with the systemd services (as returned by
        # sd_mock) and thus not be returned by _get_sysv_services(). It doesn't
        # matter that it's not part of the _LIST_UNIT_FILES output, we just
        # want to ensure that 'foo' isn't identified as a disabled initscript
        # even though below we are mocking it to show as not enabled (since
        # only 'baz' will be considered an enabled sysv service).
        listdir_mock = MagicMock(return_value=["foo", "bar", "baz", "README"])
        sd_mock = MagicMock(
            return_value=set([x.replace(".service", "") for x in _SYSTEMCTL_STATUS])
        )
        access_mock = MagicMock(
            side_effect=lambda x, y: x
            != os.path.join(systemd.INITSCRIPT_PATH, "README")
        )
        sysv_enabled_mock = MagicMock(side_effect=lambda x, _: x == "baz")

        with patch.dict(systemd.__salt__, {"cmd.run": cmd_mock}):
            with patch.object(os, "listdir", listdir_mock):
                with patch.object(systemd, "_get_systemd_services", sd_mock):
                    with patch.object(os, "access", side_effect=access_mock):
                        with patch.object(systemd, "_sysv_enabled", sysv_enabled_mock):
                            self.assertListEqual(
                                systemd.get_disabled(),
                                ["bar", "service2", "timer2.timer"],
                            )

    def test_get_all(self):
        """
        Test to return a list of all available services
        """
        listdir_mock = MagicMock(
            side_effect=[
                ["foo.service", "multi-user.target.wants", "mytimer.timer"],
                [],
                ["foo.service", "multi-user.target.wants", "bar.service"],
                ["mysql", "nginx", "README"],
            ]
        )
        access_mock = MagicMock(
            side_effect=lambda x, y: x
            != os.path.join(systemd.INITSCRIPT_PATH, "README")
        )
        with patch.object(os, "listdir", listdir_mock):
            with patch.object(os, "access", side_effect=access_mock):
                self.assertListEqual(
                    systemd.get_all(), ["bar", "foo", "mysql", "mytimer.timer", "nginx"]
                )

    def test_available(self):
        """
        Test to check that the given service is available
        """
        mock = MagicMock(side_effect=lambda x: _SYSTEMCTL_STATUS[x])

        # systemd < 231
        with patch.dict(systemd.__context__, {"salt.utils.systemd.version": 230}):
            with patch.object(systemd, "_systemctl_status", mock):
                self.assertTrue(systemd.available("sshd.service"))
                self.assertFalse(systemd.available("foo.service"))

        # systemd >= 231
        with patch.dict(systemd.__context__, {"salt.utils.systemd.version": 231}):
            with patch.dict(_SYSTEMCTL_STATUS, _SYSTEMCTL_STATUS_GTE_231):
                with patch.object(systemd, "_systemctl_status", mock):
                    self.assertTrue(systemd.available("sshd.service"))
                    self.assertFalse(systemd.available("bar.service"))

        # systemd < 231 with retcode/output changes backported (e.g. RHEL 7.3)
        with patch.dict(systemd.__context__, {"salt.utils.systemd.version": 219}):
            with patch.dict(_SYSTEMCTL_STATUS, _SYSTEMCTL_STATUS_GTE_231):
                with patch.object(systemd, "_systemctl_status", mock):
                    self.assertTrue(systemd.available("sshd.service"))
                    self.assertFalse(systemd.available("bar.service"))

    def test_missing(self):
        """
            Test to the inverse of service.available.
        """
        mock = MagicMock(side_effect=lambda x: _SYSTEMCTL_STATUS[x])

        # systemd < 231
        with patch.dict(systemd.__context__, {"salt.utils.systemd.version": 230}):
            with patch.object(systemd, "_systemctl_status", mock):
                self.assertFalse(systemd.missing("sshd.service"))
                self.assertTrue(systemd.missing("foo.service"))

        # systemd >= 231
        with patch.dict(systemd.__context__, {"salt.utils.systemd.version": 231}):
            with patch.dict(_SYSTEMCTL_STATUS, _SYSTEMCTL_STATUS_GTE_231):
                with patch.object(systemd, "_systemctl_status", mock):
                    self.assertFalse(systemd.missing("sshd.service"))
                    self.assertTrue(systemd.missing("bar.service"))

        # systemd < 231 with retcode/output changes backported (e.g. RHEL 7.3)
        with patch.dict(systemd.__context__, {"salt.utils.systemd.version": 219}):
            with patch.dict(_SYSTEMCTL_STATUS, _SYSTEMCTL_STATUS_GTE_231):
                with patch.object(systemd, "_systemctl_status", mock):
                    self.assertFalse(systemd.missing("sshd.service"))
                    self.assertTrue(systemd.missing("bar.service"))

    def test_show(self):
        """
        Test to show properties of one or more units/jobs or the manager
        """
        show_output = "a=b\nc=d\ne={ f=g ; h=i }\nWants=foo.service bar.service\n"
        mock = MagicMock(return_value=show_output)
        with patch.dict(systemd.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(
                systemd.show("sshd"),
                {
                    "a": "b",
                    "c": "d",
                    "e": {"f": "g", "h": "i"},
                    "Wants": ["foo.service", "bar.service"],
                },
            )

    def test_execs(self):
        """
        Test to return a list of all files specified as ``ExecStart`` for all
        services
        """
        mock = MagicMock(return_value=["a", "b"])
        with patch.object(systemd, "get_all", mock):
            mock = MagicMock(return_value={"ExecStart": {"path": "c"}})
            with patch.object(systemd, "show", mock):
                self.assertDictEqual(systemd.execs(), {"a": "c", "b": "c"})


class SystemdScopeTestCase(TestCase, LoaderModuleMockMixin):
    """
        Test case for salt.modules.systemd, for functions which use systemd
        scopes
    """

    def setup_loader_modules(self):
        return {systemd: {}}

    unit_name = "foo"
    mock_none = MagicMock(return_value=None)
    mock_success = MagicMock(return_value=0)
    mock_failure = MagicMock(return_value=1)
    mock_true = MagicMock(return_value=True)
    mock_false = MagicMock(return_value=False)
    mock_empty_list = MagicMock(return_value=[])
    mock_run_all_success = MagicMock(
        return_value={"retcode": 0, "stdout": "", "stderr": "", "pid": 12345}
    )
    mock_run_all_failure = MagicMock(
        return_value={"retcode": 1, "stdout": "", "stderr": "", "pid": 12345}
    )

    def _change_state(self, action, no_block=False):
        """
        Common code for start/stop/restart/reload/force_reload tests
        """
        # We want the traceback if the function name can't be found in the
        # systemd execution module.
        func = getattr(systemd, action)
        # Remove trailing _ in "reload_"
        action = action.rstrip("_").replace("_", "-")
        systemctl_command = ["systemctl"]
        if no_block:
            systemctl_command.append("--no-block")
        systemctl_command.extend([action, self.unit_name + ".service"])
        scope_prefix = ["systemd-run", "--scope"]

        assert_kwargs = {"python_shell": False}
        if action in ("enable", "disable"):
            assert_kwargs["ignore_retcode"] = True

        with patch.object(systemd, "_check_for_unit_changes", self.mock_none):
            with patch.object(systemd, "_unit_file_changed", self.mock_none):
                with patch.object(systemd, "_check_unmask", self.mock_none):
                    with patch.object(
                        systemd, "_get_sysv_services", self.mock_empty_list
                    ):

                        # Has scopes available
                        with patch.object(
                            salt.utils.systemd, "has_scope", self.mock_true
                        ):

                            # Scope enabled, successful
                            with patch.dict(
                                systemd.__salt__,
                                {
                                    "config.get": self.mock_true,
                                    "cmd.run_all": self.mock_run_all_success,
                                },
                            ):
                                ret = func(self.unit_name, no_block=no_block)
                                self.assertTrue(ret)
                                self.mock_run_all_success.assert_called_with(
                                    scope_prefix + systemctl_command, **assert_kwargs
                                )

                            # Scope enabled, failed
                            with patch.dict(
                                systemd.__salt__,
                                {
                                    "config.get": self.mock_true,
                                    "cmd.run_all": self.mock_run_all_failure,
                                },
                            ):
                                if action in ("stop", "disable"):
                                    ret = func(self.unit_name, no_block=no_block)
                                    self.assertFalse(ret)
                                else:
                                    self.assertRaises(
                                        CommandExecutionError,
                                        func,
                                        self.unit_name,
                                        no_block=no_block,
                                    )
                                self.mock_run_all_failure.assert_called_with(
                                    scope_prefix + systemctl_command, **assert_kwargs
                                )

                            # Scope disabled, successful
                            with patch.dict(
                                systemd.__salt__,
                                {
                                    "config.get": self.mock_false,
                                    "cmd.run_all": self.mock_run_all_success,
                                },
                            ):
                                ret = func(self.unit_name, no_block=no_block)
                                self.assertTrue(ret)
                                self.mock_run_all_success.assert_called_with(
                                    systemctl_command, **assert_kwargs
                                )

                            # Scope disabled, failed
                            with patch.dict(
                                systemd.__salt__,
                                {
                                    "config.get": self.mock_false,
                                    "cmd.run_all": self.mock_run_all_failure,
                                },
                            ):
                                if action in ("stop", "disable"):
                                    ret = func(self.unit_name, no_block=no_block)
                                    self.assertFalse(ret)
                                else:
                                    self.assertRaises(
                                        CommandExecutionError,
                                        func,
                                        self.unit_name,
                                        no_block=no_block,
                                    )
                                self.mock_run_all_failure.assert_called_with(
                                    systemctl_command, **assert_kwargs
                                )

                        # Does not have scopes available
                        with patch.object(
                            salt.utils.systemd, "has_scope", self.mock_false
                        ):

                            # The results should be the same irrespective of
                            # whether or not scope is enabled, since scope is not
                            # available, so we repeat the below tests with it both
                            # enabled and disabled.
                            for scope_mock in (self.mock_true, self.mock_false):

                                # Successful
                                with patch.dict(
                                    systemd.__salt__,
                                    {
                                        "config.get": scope_mock,
                                        "cmd.run_all": self.mock_run_all_success,
                                    },
                                ):
                                    ret = func(self.unit_name, no_block=no_block)
                                    self.assertTrue(ret)
                                    self.mock_run_all_success.assert_called_with(
                                        systemctl_command, **assert_kwargs
                                    )

                                # Failed
                                with patch.dict(
                                    systemd.__salt__,
                                    {
                                        "config.get": scope_mock,
                                        "cmd.run_all": self.mock_run_all_failure,
                                    },
                                ):
                                    if action in ("stop", "disable"):
                                        ret = func(self.unit_name, no_block=no_block)
                                        self.assertFalse(ret)
                                    else:
                                        self.assertRaises(
                                            CommandExecutionError,
                                            func,
                                            self.unit_name,
                                            no_block=no_block,
                                        )
                                    self.mock_run_all_failure.assert_called_with(
                                        systemctl_command, **assert_kwargs
                                    )

    def _mask_unmask(self, action, runtime):
        """
        Common code for mask/unmask tests
        """
        # We want the traceback if the function name can't be found in the
        # systemd execution module, so don't provide a fallback value for the
        # call to getattr() here.
        func = getattr(systemd, action)
        # Remove trailing _ in "unmask_"
        action = action.rstrip("_").replace("_", "-")
        systemctl_command = ["systemctl", action]
        if runtime:
            systemctl_command.append("--runtime")
        systemctl_command.append(self.unit_name + ".service")
        scope_prefix = ["systemd-run", "--scope"]

        args = [self.unit_name, runtime]

        masked_mock = self.mock_true if action == "unmask" else self.mock_false

        with patch.object(systemd, "_check_for_unit_changes", self.mock_none):
            if action == "unmask":
                mock_not_run = MagicMock(
                    return_value={
                        "retcode": 0,
                        "stdout": "",
                        "stderr": "",
                        "pid": 12345,
                    }
                )
                with patch.dict(systemd.__salt__, {"cmd.run_all": mock_not_run}):
                    with patch.object(systemd, "masked", self.mock_false):
                        # Test not masked (should take no action and return True)
                        self.assertTrue(systemd.unmask_(self.unit_name))
                        # Also should not have called cmd.run_all
                        self.assertTrue(mock_not_run.call_count == 0)

            with patch.object(systemd, "masked", masked_mock):

                # Has scopes available
                with patch.object(salt.utils.systemd, "has_scope", self.mock_true):

                    # Scope enabled, successful
                    with patch.dict(
                        systemd.__salt__,
                        {
                            "config.get": self.mock_true,
                            "cmd.run_all": self.mock_run_all_success,
                        },
                    ):
                        ret = func(*args)
                        self.assertTrue(ret)
                        self.mock_run_all_success.assert_called_with(
                            scope_prefix + systemctl_command,
                            python_shell=False,
                            redirect_stderr=True,
                        )

                    # Scope enabled, failed
                    with patch.dict(
                        systemd.__salt__,
                        {
                            "config.get": self.mock_true,
                            "cmd.run_all": self.mock_run_all_failure,
                        },
                    ):
                        self.assertRaises(CommandExecutionError, func, *args)
                        self.mock_run_all_failure.assert_called_with(
                            scope_prefix + systemctl_command,
                            python_shell=False,
                            redirect_stderr=True,
                        )

                    # Scope disabled, successful
                    with patch.dict(
                        systemd.__salt__,
                        {
                            "config.get": self.mock_false,
                            "cmd.run_all": self.mock_run_all_success,
                        },
                    ):
                        ret = func(*args)
                        self.assertTrue(ret)
                        self.mock_run_all_success.assert_called_with(
                            systemctl_command, python_shell=False, redirect_stderr=True
                        )

                    # Scope disabled, failed
                    with patch.dict(
                        systemd.__salt__,
                        {
                            "config.get": self.mock_false,
                            "cmd.run_all": self.mock_run_all_failure,
                        },
                    ):
                        self.assertRaises(CommandExecutionError, func, *args)
                        self.mock_run_all_failure.assert_called_with(
                            systemctl_command, python_shell=False, redirect_stderr=True
                        )

                # Does not have scopes available
                with patch.object(salt.utils.systemd, "has_scope", self.mock_false):

                    # The results should be the same irrespective of
                    # whether or not scope is enabled, since scope is not
                    # available, so we repeat the below tests with it both
                    # enabled and disabled.
                    for scope_mock in (self.mock_true, self.mock_false):

                        # Successful
                        with patch.dict(
                            systemd.__salt__,
                            {
                                "config.get": scope_mock,
                                "cmd.run_all": self.mock_run_all_success,
                            },
                        ):
                            ret = func(*args)
                            self.assertTrue(ret)
                            self.mock_run_all_success.assert_called_with(
                                systemctl_command,
                                python_shell=False,
                                redirect_stderr=True,
                            )

                        # Failed
                        with patch.dict(
                            systemd.__salt__,
                            {
                                "config.get": scope_mock,
                                "cmd.run_all": self.mock_run_all_failure,
                            },
                        ):
                            self.assertRaises(CommandExecutionError, func, *args)
                            self.mock_run_all_failure.assert_called_with(
                                systemctl_command,
                                python_shell=False,
                                redirect_stderr=True,
                            )

    def test_start(self):
        self._change_state("start", no_block=False)
        self._change_state("start", no_block=True)

    def test_stop(self):
        self._change_state("stop", no_block=False)
        self._change_state("stop", no_block=True)

    def test_restart(self):
        self._change_state("restart", no_block=False)
        self._change_state("restart", no_block=True)

    def test_reload(self):
        self._change_state("reload_", no_block=False)
        self._change_state("reload_", no_block=True)

    def test_force_reload(self):
        self._change_state("force_reload", no_block=False)
        self._change_state("force_reload", no_block=True)

    def test_enable(self):
        self._change_state("enable", no_block=False)
        self._change_state("enable", no_block=True)

    def test_disable(self):
        self._change_state("disable", no_block=False)
        self._change_state("disable", no_block=True)

    def test_mask(self):
        self._mask_unmask("mask", False)

    def test_mask_runtime(self):
        self._mask_unmask("mask", True)

    def test_unmask(self):
        self._mask_unmask("unmask_", False)

    def test_unmask_runtime(self):
        self._mask_unmask("unmask_", True)

    def test_firstboot(self):
        """
        Test service.firstboot without parameters
        """
        result = {"retcode": 0, "stdout": "stdout"}
        salt_mock = {
            "cmd.run_all": MagicMock(return_value=result),
        }
        with patch.dict(systemd.__salt__, salt_mock):
            assert systemd.firstboot()
            salt_mock["cmd.run_all"].assert_called_with(["systemd-firstboot"])

    def test_firstboot_params(self):
        """
        Test service.firstboot with parameters
        """
        result = {"retcode": 0, "stdout": "stdout"}
        salt_mock = {
            "cmd.run_all": MagicMock(return_value=result),
        }
        with patch.dict(systemd.__salt__, salt_mock):
            assert systemd.firstboot(
                locale="en_US.UTF-8",
                locale_message="en_US.UTF-8",
                keymap="jp",
                timezone="Europe/Berlin",
                hostname="node-001",
                machine_id="1234567890abcdef",
                root="/mnt",
            )
            salt_mock["cmd.run_all"].assert_called_with(
                [
                    "systemd-firstboot",
                    "--locale",
                    "en_US.UTF-8",
                    "--locale-message",
                    "en_US.UTF-8",
                    "--keymap",
                    "jp",
                    "--timezone",
                    "Europe/Berlin",
                    "--hostname",
                    "node-001",
                    "--machine-ID",
                    "1234567890abcdef",
                    "--root",
                    "/mnt",
                ]
            )

    def test_firstboot_error(self):
        """
        Test service.firstboot error
        """
        result = {"retcode": 1, "stderr": "error"}
        salt_mock = {
            "cmd.run_all": MagicMock(return_value=result),
        }
        with patch.dict(systemd.__salt__, salt_mock):
            with pytest.raises(CommandExecutionError):
                assert systemd.firstboot()



# -*- coding: utf-8 -*-
"""
virt.query outputter
====================

Used to display the output from the :mod:`virt.query <salt.runners.virt.query>`
runner.
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import 3rd-party libs
from salt.ext import six


def output(data, **kwargs):  # pylint: disable=unused-argument
    """
    Display output for the salt-run virt.query function
    """
    out = ""
    for id_ in data["data"]:
        out += "{0}\n".format(id_)
        for vm_ in data["data"][id_]["vm_info"]:
            out += "  {0}\n".format(vm_)
            vm_data = data[id_]["vm_info"][vm_]
            if "cpu" in vm_data:
                out += "    CPU: {0}\n".format(vm_data["cpu"])
            if "mem" in vm_data:
                out += "    Memory: {0}\n".format(vm_data["mem"])
            if "state" in vm_data:
                out += "    State: {0}\n".format(vm_data["state"])
            if "graphics" in vm_data:
                if vm_data["graphics"].get("type", "") == "vnc":
                    out += "    Graphics: vnc - {0}:{1}\n".format(
                        id_, vm_data["graphics"]["port"]
                    )
            if "disks" in vm_data:
                for disk, d_data in six.iteritems(vm_data["disks"]):
                    out += "    Disk - {0}:\n".format(disk)
                    out += "      Size: {0}\n".format(d_data["disk size"])
                    out += "      File: {0}\n".format(d_data["file"])
                    out += "      File Format: {0}\n".format(d_data["file format"])
            if "nics" in vm_data:
                for mac in vm_data["nics"]:
                    out += "    Nic - {0}:\n".format(mac)
                    out += "      Source: {0}\n".format(
                        vm_data["nics"][mac]["source"][
                            next(six.iterkeys(vm_data["nics"][mac]["source"]))
                        ]
                    )
                    out += "      Type: {0}\n".format(vm_data["nics"][mac]["type"])
    return out



# -*- coding: utf-8 -*-
# pylint: disable=unused-argument

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.states.jboss7 as jboss7
from salt.exceptions import CommandExecutionError
from salt.ext import six

# Import Salt testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class JBoss7StateTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {
            jboss7: {
                "__salt__": {
                    "jboss7.read_datasource": MagicMock(),
                    "jboss7.create_datasource": MagicMock(),
                    "jboss7.update_datasource": MagicMock(),
                    "jboss7.remove_datasource": MagicMock(),
                    "jboss7.read_simple_binding": MagicMock(),
                    "jboss7.create_simple_binding": MagicMock(),
                    "jboss7.update_simple_binding": MagicMock(),
                    "jboss7.undeploy": MagicMock(),
                    "jboss7.deploy": MagicMock,
                    "file.get_managed": MagicMock,
                    "file.manage_file": MagicMock,
                    "jboss7.list_deployments": MagicMock,
                },
                "__env__": "base",
            }
        }

    def test_should_not_redeploy_unchanged(self):
        # given
        parameters = {
            "target_file": "some_artifact",
            "undeploy_force": False,
            "undeploy": "some_artifact",
            "source": "some_artifact_on_master",
        }
        jboss_conf = {"cli_path": "somewhere", "controller": "some_controller"}

        def list_deployments(jboss_config):
            return ["some_artifact"]

        def file_get_managed(
            name,
            template,
            source,
            source_hash,
            source_hash_name,
            user,
            group,
            mode,
            attrs,
            saltenv,
            context,
            defaults,
            skip_verify,
            kwargs,
        ):
            return "sfn", "hash", ""

        def file_manage_file(
            name,
            sfn,
            ret,
            source,
            source_sum,
            user,
            group,
            mode,
            attrs,
            saltenv,
            backup,
            makedirs,
            template,
            show_diff,
            contents,
            dir_mode,
        ):
            return {"result": True, "changes": False}

        jboss7_undeploy_mock = MagicMock()
        jboss7_deploy_mock = MagicMock()
        file_get_managed = MagicMock(side_effect=file_get_managed)
        file_manage_file = MagicMock(side_effect=file_manage_file)
        list_deployments_mock = MagicMock(side_effect=list_deployments)
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.undeploy": jboss7_undeploy_mock,
                "jboss7.deploy": jboss7_deploy_mock,
                "file.get_managed": file_get_managed,
                "file.manage_file": file_manage_file,
                "jboss7.list_deployments": list_deployments_mock,
            },
        ):
            # when
            result = jboss7.deployed(
                name="unchanged", jboss_config=jboss_conf, salt_source=parameters
            )

            # then
            self.assertFalse(jboss7_undeploy_mock.called)
            self.assertFalse(jboss7_deploy_mock.called)

    def test_should_redeploy_changed(self):
        # given
        parameters = {
            "target_file": "some_artifact",
            "undeploy_force": False,
            "undeploy": "some_artifact",
            "source": "some_artifact_on_master",
        }
        jboss_conf = {"cli_path": "somewhere", "controller": "some_controller"}

        def list_deployments(jboss_config):
            return ["some_artifact"]

        def file_get_managed(
            name,
            template,
            source,
            source_hash,
            source_hash_name,
            user,
            group,
            mode,
            attrs,
            saltenv,
            context,
            defaults,
            skip_verify,
            kwargs,
        ):
            return "sfn", "hash", ""

        def file_manage_file(
            name,
            sfn,
            ret,
            source,
            source_sum,
            user,
            group,
            mode,
            attrs,
            saltenv,
            backup,
            makedirs,
            template,
            show_diff,
            contents,
            dir_mode,
        ):
            return {"result": True, "changes": True}

        jboss7_undeploy_mock = MagicMock()
        jboss7_deploy_mock = MagicMock()
        file_get_managed = MagicMock(side_effect=file_get_managed)
        file_manage_file = MagicMock(side_effect=file_manage_file)
        list_deployments_mock = MagicMock(side_effect=list_deployments)
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.undeploy": jboss7_undeploy_mock,
                "jboss7.deploy": jboss7_deploy_mock,
                "file.get_managed": file_get_managed,
                "file.manage_file": file_manage_file,
                "jboss7.list_deployments": list_deployments_mock,
            },
        ):
            # when
            result = jboss7.deployed(
                name="unchanged", jboss_config=jboss_conf, salt_source=parameters
            )

            # then
            self.assertTrue(jboss7_undeploy_mock.called)
            self.assertTrue(jboss7_deploy_mock.called)

    def test_should_deploy_different_artifact(self):
        # given
        parameters = {
            "target_file": "some_artifact",
            "undeploy_force": False,
            "undeploy": "some_artifact",
            "source": "some_artifact_on_master",
        }
        jboss_conf = {"cli_path": "somewhere", "controller": "some_controller"}

        def list_deployments(jboss_config):
            return ["some_other_artifact"]

        def file_get_managed(
            name,
            template,
            source,
            source_hash,
            source_hash_name,
            user,
            group,
            mode,
            attrs,
            saltenv,
            context,
            defaults,
            skip_verify,
            kwargs,
        ):
            return "sfn", "hash", ""

        def file_manage_file(
            name,
            sfn,
            ret,
            source,
            source_sum,
            user,
            group,
            mode,
            attrs,
            saltenv,
            backup,
            makedirs,
            template,
            show_diff,
            contents,
            dir_mode,
        ):
            return {"result": True, "changes": False}

        jboss7_undeploy_mock = MagicMock()
        jboss7_deploy_mock = MagicMock()
        file_get_managed = MagicMock(side_effect=file_get_managed)
        file_manage_file = MagicMock(side_effect=file_manage_file)
        list_deployments_mock = MagicMock(side_effect=list_deployments)
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.undeploy": jboss7_undeploy_mock,
                "jboss7.deploy": jboss7_deploy_mock,
                "file.get_managed": file_get_managed,
                "file.manage_file": file_manage_file,
                "jboss7.list_deployments": list_deployments_mock,
            },
        ):
            # when
            result = jboss7.deployed(
                name="unchanged", jboss_config=jboss_conf, salt_source=parameters
            )

            # then
            self.assertFalse(jboss7_undeploy_mock.called)
            self.assertTrue(jboss7_deploy_mock.called)

    def test_should_redploy_undeploy_force(self):
        # given
        parameters = {
            "target_file": "some_artifact",
            "undeploy_force": True,
            "undeploy": "some_artifact",
            "source": "some_artifact_on_master",
        }
        jboss_conf = {"cli_path": "somewhere", "controller": "some_controller"}

        def list_deployments(jboss_config):
            return ["some_artifact"]

        def file_get_managed(
            name,
            template,
            source,
            source_hash,
            source_hash_name,
            user,
            group,
            mode,
            attrs,
            saltenv,
            context,
            defaults,
            skip_verify,
            kwargs,
        ):
            return "sfn", "hash", ""

        def file_manage_file(
            name,
            sfn,
            ret,
            source,
            source_sum,
            user,
            group,
            mode,
            attrs,
            saltenv,
            backup,
            makedirs,
            template,
            show_diff,
            contents,
            dir_mode,
        ):
            return {"result": True, "changes": False}

        jboss7_undeploy_mock = MagicMock()
        jboss7_deploy_mock = MagicMock()
        file_get_managed = MagicMock(side_effect=file_get_managed)
        file_manage_file = MagicMock(side_effect=file_manage_file)
        list_deployments_mock = MagicMock(side_effect=list_deployments)
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.undeploy": jboss7_undeploy_mock,
                "jboss7.deploy": jboss7_deploy_mock,
                "file.get_managed": file_get_managed,
                "file.manage_file": file_manage_file,
                "jboss7.list_deployments": list_deployments_mock,
            },
        ):
            # when
            result = jboss7.deployed(
                name="unchanged", jboss_config=jboss_conf, salt_source=parameters
            )

            # then
            self.assertTrue(jboss7_undeploy_mock.called)
            self.assertTrue(jboss7_deploy_mock.called)

    def test_should_create_new_datasource_if_not_exists(self):
        # given
        datasource_properties = {"connection-url": "jdbc:/old-connection-url"}
        ds_status = {"created": False}

        def read_func(jboss_config, name, profile):
            if ds_status["created"]:
                return {"success": True, "result": datasource_properties}
            else:
                return {"success": False, "err_code": "JBAS014807"}

        def create_func(jboss_config, name, datasource_properties, profile):
            ds_status["created"] = True
            return {"success": True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock(side_effect=create_func)
        update_mock = MagicMock()
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_datasource": read_mock,
                "jboss7.create_datasource": create_mock,
                "jboss7.update_datasource": update_mock,
            },
        ):

            # when
            result = jboss7.datasource_exists(
                name="appDS",
                jboss_config={},
                datasource_properties=datasource_properties,
                profile=None,
            )

            # then
            create_mock.assert_called_with(
                name="appDS",
                jboss_config={},
                datasource_properties=datasource_properties,
                profile=None,
            )

            self.assertFalse(update_mock.called)
            self.assertEqual(result["comment"], "Datasource created.")

    def test_should_update_the_datasource_if_exists(self):
        ds_status = {"updated": False}

        def read_func(jboss_config, name, profile):
            if ds_status["updated"]:
                return {
                    "success": True,
                    "result": {"connection-url": "jdbc:/new-connection-url"},
                }
            else:
                return {
                    "success": True,
                    "result": {"connection-url": "jdbc:/old-connection-url"},
                }

        def update_func(jboss_config, name, new_properties, profile):
            ds_status["updated"] = True
            return {"success": True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock()
        update_mock = MagicMock(side_effect=update_func)
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_datasource": read_mock,
                "jboss7.create_datasource": create_mock,
                "jboss7.update_datasource": update_mock,
            },
        ):
            result = jboss7.datasource_exists(
                name="appDS",
                jboss_config={},
                datasource_properties={"connection-url": "jdbc:/new-connection-url"},
                profile=None,
            )

            update_mock.assert_called_with(
                name="appDS",
                jboss_config={},
                new_properties={"connection-url": "jdbc:/new-connection-url"},
                profile=None,
            )
            self.assertTrue(read_mock.called)
            self.assertEqual(result["comment"], "Datasource updated.")

    def test_should_recreate_the_datasource_if_specified(self):
        read_mock = MagicMock(
            return_value={
                "success": True,
                "result": {"connection-url": "jdbc:/same-connection-url"},
            }
        )
        create_mock = MagicMock(return_value={"success": True})
        remove_mock = MagicMock(return_value={"success": True})
        update_mock = MagicMock()
        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_datasource": read_mock,
                "jboss7.create_datasource": create_mock,
                "jboss7.remove_datasource": remove_mock,
                "jboss7.update_datasource": update_mock,
            },
        ):
            result = jboss7.datasource_exists(
                name="appDS",
                jboss_config={},
                datasource_properties={"connection-url": "jdbc:/same-connection-url"},
                recreate=True,
            )

            remove_mock.assert_called_with(name="appDS", jboss_config={}, profile=None)
            create_mock.assert_called_with(
                name="appDS",
                jboss_config={},
                datasource_properties={"connection-url": "jdbc:/same-connection-url"},
                profile=None,
            )
            self.assertEqual(result["changes"]["removed"], "appDS")
            self.assertEqual(result["changes"]["created"], "appDS")

    def test_should_inform_if_the_datasource_has_not_changed(self):
        read_mock = MagicMock(
            return_value={
                "success": True,
                "result": {"connection-url": "jdbc:/same-connection-url"},
            }
        )
        create_mock = MagicMock()
        remove_mock = MagicMock()
        update_mock = MagicMock(return_value={"success": True})

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_datasource": read_mock,
                "jboss7.create_datasource": create_mock,
                "jboss7.remove_datasource": remove_mock,
                "jboss7.update_datasource": update_mock,
            },
        ):
            result = jboss7.datasource_exists(
                name="appDS",
                jboss_config={},
                datasource_properties={"connection-url": "jdbc:/old-connection-url"},
            )

            update_mock.assert_called_with(
                name="appDS",
                jboss_config={},
                new_properties={"connection-url": "jdbc:/old-connection-url"},
                profile=None,
            )
            self.assertFalse(create_mock.called)
            self.assertEqual(result["comment"], "Datasource not changed.")

    def test_should_create_binding_if_not_exists(self):
        # given
        binding_status = {"created": False}

        def read_func(jboss_config, binding_name, profile):
            if binding_status["created"]:
                return {"success": True, "result": {"value": "DEV"}}
            else:
                return {"success": False, "err_code": "JBAS014807"}

        def create_func(jboss_config, binding_name, value, profile):
            binding_status["created"] = True
            return {"success": True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock(side_effect=create_func)
        update_mock = MagicMock()

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_simple_binding": read_mock,
                "jboss7.create_simple_binding": create_mock,
                "jboss7.update_simple_binding": update_mock,
            },
        ):

            # when
            result = jboss7.bindings_exist(
                name="bindings", jboss_config={}, bindings={"env": "DEV"}, profile=None
            )

            # then
            create_mock.assert_called_with(
                jboss_config={}, binding_name="env", value="DEV", profile=None
            )
            self.assertEqual(update_mock.call_count, 0)
            self.assertEqual(result["changes"], {"added": "env:DEV\n"})
            self.assertEqual(result["comment"], "Bindings changed.")

    def test_should_update_bindings_if_exists_and_different(self):
        # given
        binding_status = {"updated": False}

        def read_func(jboss_config, binding_name, profile):
            if binding_status["updated"]:
                return {"success": True, "result": {"value": "DEV2"}}
            else:
                return {"success": True, "result": {"value": "DEV"}}

        def update_func(jboss_config, binding_name, value, profile):
            binding_status["updated"] = True
            return {"success": True}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock()
        update_mock = MagicMock(side_effect=update_func)

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_simple_binding": read_mock,
                "jboss7.create_simple_binding": create_mock,
                "jboss7.update_simple_binding": update_mock,
            },
        ):
            # when
            result = jboss7.bindings_exist(
                name="bindings", jboss_config={}, bindings={"env": "DEV2"}, profile=None
            )

            # then
            update_mock.assert_called_with(
                jboss_config={}, binding_name="env", value="DEV2", profile=None
            )
            self.assertEqual(create_mock.call_count, 0)
            self.assertEqual(result["changes"], {"changed": "env:DEV->DEV2\n"})
            self.assertEqual(result["comment"], "Bindings changed.")

    def test_should_not_update_bindings_if_same(self):
        # given
        read_mock = MagicMock(
            return_value={"success": True, "result": {"value": "DEV2"}}
        )
        create_mock = MagicMock()
        update_mock = MagicMock()

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_simple_binding": read_mock,
                "jboss7.create_simple_binding": create_mock,
                "jboss7.update_simple_binding": update_mock,
            },
        ):
            # when
            result = jboss7.bindings_exist(
                name="bindings", jboss_config={}, bindings={"env": "DEV2"}
            )

            # then
            self.assertEqual(create_mock.call_count, 0)
            self.assertEqual(update_mock.call_count, 0)
            self.assertEqual(result["changes"], {})
            self.assertEqual(result["comment"], "Bindings not changed.")

    def test_should_raise_exception_if_cannot_create_binding(self):
        def read_func(jboss_config, binding_name, profile):
            return {"success": False, "err_code": "JBAS014807"}

        def create_func(jboss_config, binding_name, value, profile):
            return {"success": False, "failure-description": "Incorrect binding name."}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock(side_effect=create_func)
        update_mock = MagicMock()

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_simple_binding": read_mock,
                "jboss7.create_simple_binding": create_mock,
                "jboss7.update_simple_binding": update_mock,
            },
        ):
            # when
            try:
                jboss7.bindings_exist(
                    name="bindings",
                    jboss_config={},
                    bindings={"env": "DEV2"},
                    profile=None,
                )
                self.fail("An exception should be thrown")
            except CommandExecutionError as e:
                self.assertEqual(six.text_type(e), "Incorrect binding name.")

    def test_should_raise_exception_if_cannot_update_binding(self):
        def read_func(jboss_config, binding_name, profile):
            return {"success": True, "result": {"value": "DEV"}}

        def update_func(jboss_config, binding_name, value, profile):
            return {"success": False, "failure-description": "Incorrect binding name."}

        read_mock = MagicMock(side_effect=read_func)
        create_mock = MagicMock()
        update_mock = MagicMock(side_effect=update_func)

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_simple_binding": read_mock,
                "jboss7.create_simple_binding": create_mock,
                "jboss7.update_simple_binding": update_mock,
            },
        ):

            # when
            try:
                jboss7.bindings_exist(
                    name="bindings",
                    jboss_config={},
                    bindings={"env": "!@#!///some weird value"},
                    profile=None,
                )
                self.fail("An exception should be thrown")
            except CommandExecutionError as e:
                self.assertEqual(six.text_type(e), "Incorrect binding name.")

    def test_datasource_exist_create_datasource_good_code(self):
        jboss_config = {
            "cli_path": "/home/ch44d/Desktop/wildfly-18.0.0.Final/bin/jboss-cli.sh",
            "controller": "127.0.0.1: 9990",
            "cli_user": "user",
            "cli_password": "user",
        }

        datasource_properties = {
            "driver - name": "h2",
            "connection - url": "jdbc:sqlserver://127.0.0.1:1433;DatabaseName=test_s2",
            "jndi - name": "java:/home/ch44d/Desktop/sqljdbc_7.4/enu/mssql-jdbc-7.4.1.jre8.jar",
            "user - name": "user",
            "password": "user",
            "use - java - context": True,
        }

        read_datasource = MagicMock(
            return_value={"success": False, "err_code": "WFLYCTL0216"}
        )

        error_msg = "Error: -1"
        create_datasource = MagicMock(
            return_value={"success": False, "stdout": error_msg}
        )

        with patch.dict(
            jboss7.__salt__,
            {
                "jboss7.read_datasource": read_datasource,
                "jboss7.create_datasource": create_datasource,
            },
        ):
            ret = jboss7.datasource_exists("SQL", jboss_config, datasource_properties)

            self.assertTrue("result" in ret)
            self.assertFalse(ret["result"])
            self.assertTrue("comment" in ret)
            self.assertTrue(error_msg in ret["comment"])

            read_datasource.assert_called_once()
            create_datasource.assert_called_once()

    def test_datasource_exist_create_datasource_bad_code(self):
        jboss_config = {
            "cli_path": "/home/ch44d/Desktop/wildfly-18.0.0.Final/bin/jboss-cli.sh",
            "controller": "127.0.0.1: 9990",
            "cli_user": "user",
            "cli_password": "user",
        }

        datasource_properties = {
            "driver - name": "h2",
            "connection - url": "jdbc:sqlserver://127.0.0.1:1433;DatabaseName=test_s2",
            "jndi - name": "java:/home/ch44d/Desktop/sqljdbc_7.4/enu/mssql-jdbc-7.4.1.jre8.jar",
            "user - name": "user",
            "password": "user",
            "use - java - context": True,
        }

        read_datasource = MagicMock(
            return_value={
                "success": False,
                "err_code": "WFLYCTL0217",
                "failure-description": "Something happened",
            }
        )

        with patch.dict(jboss7.__salt__, {"jboss7.read_datasource": read_datasource}):
            self.assertRaises(
                CommandExecutionError,
                jboss7.datasource_exists,
                "SQL",
                jboss_config,
                datasource_properties,
            )
            read_datasource.assert_called_once()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.portage_config as portage_config

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PortageConfigTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.portage_config
    """

    def setup_loader_modules(self):
        return {portage_config: {"__opts__": {"test": True}}}

    # 'mod_init' function tests: 1

    def test_mod_init(self):
        """
        Test to enforce a nice structure on the configuration files.
        """
        name = "salt"

        mock = MagicMock(side_effect=[True, Exception])
        with patch.dict(
            portage_config.__salt__, {"portage_config.enforce_nice_config": mock}
        ):
            self.assertTrue(portage_config.mod_init(name))

            self.assertFalse(portage_config.mod_init(name))

    # 'flags' function tests: 1

    def test_flags(self):
        """
        Test to enforce the given flags on the given package or ``DEPEND`` atom.
        """
        with patch("traceback.format_exc", MagicMock(return_value="SALTSTACK")):
            name = "salt"

            ret = {"name": name, "result": False, "comment": "SALTSTACK", "changes": {}}

            mock = MagicMock(side_effect=Exception("error"))
            with patch.dict(
                portage_config.__salt__, {"portage_config.get_missing_flags": mock}
            ):
                self.assertDictEqual(portage_config.flags(name, use="openssl"), ret)

                self.assertDictEqual(
                    portage_config.flags(name, accept_keywords=True), ret
                )

                self.assertDictEqual(portage_config.flags(name, env=True), ret)

                self.assertDictEqual(portage_config.flags(name, license=True), ret)

                self.assertDictEqual(portage_config.flags(name, properties=True), ret)

                self.assertDictEqual(portage_config.flags(name, mask=True), ret)

                self.assertDictEqual(portage_config.flags(name, unmask=True), ret)

                ret.update({"comment": "", "result": True})
                self.assertDictEqual(portage_config.flags(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.ports as ports
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MockModule(object):
    """
    Mock of __module__
    """

    __module__ = "A"


class MockContext(object):
    """
    Mock of __context__
    """

    __context__ = {"ports.install_error": "salt"}


class MockSys(object):
    """
    Mock of sys
    """

    def __init__(self):
        self.modules = {"A": MockContext()}


class PortsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.ports
    """

    def setup_loader_modules(self):
        return {ports: {"sys": MockSys()}}

    # 'installed' function tests: 1

    def test_installed(self):
        """
        Test to verify that the desired port is installed,
        and that it was compiled with the desired options.
        """
        name = "security/nmap"
        options = [{"IPV6": "on"}]

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=SaltInvocationError)
        with patch.dict(ports.__salt__, {"ports.showconfig": mock}):
            comt = (
                "Unable to get configuration for {0}. Port name may "
                "be invalid, or ports tree may need to be updated. "
                "Error message: ".format(name)
            )
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(ports.installed(name), ret)

        mock = MagicMock(return_value={})
        mock_lst = MagicMock(return_value={"origin": {"origin": name}})
        with patch.dict(
            ports.__salt__, {"ports.showconfig": mock, "pkg.list_pkgs": mock_lst}
        ):
            comt = "security/nmap is already installed"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(ports.installed(name), ret)

            comt = (
                "security/nmap does not have any build options,"
                " yet options were specified"
            )
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(ports.installed(name, options), ret)

            mock_dict = MagicMock(return_value={"origin": {"origin": "salt"}})
            with patch.dict(ports.__salt__, {"pkg.list_pkgs": mock_dict}):
                with patch.dict(ports.__opts__, {"test": True}):
                    comt = "{0} will be installed".format(name)
                    ret.update({"comment": comt, "result": None})
                    self.assertDictEqual(ports.installed(name), ret)

        mock = MagicMock(return_value={"salt": {"salt": "salt"}})
        mock_dict = MagicMock(return_value={"origin": {"origin": "salt"}})
        mock_f = MagicMock(return_value=False)
        mock_t = MagicMock(return_value=True)
        with patch.dict(
            ports.__salt__,
            {
                "ports.showconfig": mock,
                "pkg.list_pkgs": mock_dict,
                "ports.config": mock_f,
                "ports.rmconfig": mock_t,
            },
        ):
            with patch.dict(ports.__opts__, {"test": True}):
                comt = (
                    "The following options are not available" " for security/nmap: IPV6"
                )
                ret.update({"comment": comt, "result": False})
                self.assertDictEqual(ports.installed(name, options), ret)

                comt = (
                    "security/nmap will be installed with the " "default build options"
                )
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(ports.installed(name), ret)

            with patch.dict(ports.__opts__, {"test": False}):
                comt = "Unable to set options for security/nmap"
                ret.update({"comment": comt, "result": False})
                self.assertDictEqual(ports.installed(name, [{"salt": "salt"}]), ret)

                with patch.object(os.path, "isfile", mock_t):
                    with patch.object(os.path, "isdir", mock_t):
                        comt = "Unable to clear options for security/nmap"
                        ret.update({"comment": comt, "result": False})
                        self.assertDictEqual(ports.installed(name), ret)

                with patch.dict(
                    ports.__salt__,
                    {
                        "ports.config": mock_t,
                        "ports.install": mock_t,
                        "test.ping": MockModule(),
                    },
                ):
                    comt = "Failed to install security/nmap." " Error message:\nsalt"
                    ret.update({"comment": comt, "result": False, "changes": True})
                    self.assertDictEqual(ports.installed(name, [{"salt": "salt"}]), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

import os
import os.path
import tempfile

import salt.config

# Import Salt libs
import salt.loader
from salt.exceptions import SaltRenderError

# Import 3rd-party libs
from salt.ext import six
from salt.ext.six.moves import StringIO
from tests.support.runtests import RUNTIME_VARS

# Import Salt Testing libs
from tests.support.unit import TestCase

REQUISITES = ["require", "require_in", "use", "use_in", "watch", "watch_in"]


class StateConfigRendererTestCase(TestCase):
    def setUp(self):
        self.root_dir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        self.state_tree_dir = os.path.join(self.root_dir, "state_tree")
        self.cache_dir = os.path.join(self.root_dir, "cachedir")
        if not os.path.isdir(self.root_dir):
            os.makedirs(self.root_dir)

        if not os.path.isdir(self.state_tree_dir):
            os.makedirs(self.state_tree_dir)

        if not os.path.isdir(self.cache_dir):
            os.makedirs(self.cache_dir)
        self.config = salt.config.minion_config(None)
        self.config["root_dir"] = self.root_dir
        self.config["state_events"] = False
        self.config["id"] = "match"
        self.config["file_client"] = "local"
        self.config["file_roots"] = dict(base=[self.state_tree_dir])
        self.config["cachedir"] = self.cache_dir
        self.config["test"] = False
        self._renderers = salt.loader.render(
            self.config, {"config.get": lambda a, b: False}
        )

    def tearDown(self):
        for attrname in ("config", "_renderers"):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def _render_sls(
        self, content, sls="", saltenv="base", argline="-G yaml . jinja", **kws
    ):
        return self._renderers["stateconf"](
            StringIO(content),
            saltenv=saltenv,
            sls=sls,
            argline=argline,
            renderers=salt.loader.render(self.config, {}),
            **kws
        )

    def test_state_config(self):
        result = self._render_sls(
            """
.sls_params:
  stateconf.set:
    - name1: value1
    - name2: value2

.extra:
  stateconf:
    - set
    - name: value

# --- end of state config ---

test:
  cmd.run:
    - name: echo name1={{sls_params.name1}} name2={{sls_params.name2}} {{extra.name}}
    - cwd: /
""",
            sls="test",
        )
        self.assertEqual(len(result), 3)
        self.assertTrue("test::sls_params" in result and "test" in result)
        self.assertTrue("test::extra" in result)
        self.assertEqual(
            result["test"]["cmd.run"][0]["name"], "echo name1=value1 name2=value2 value"
        )

    def test_sls_dir(self):
        result = self._render_sls(
            """
test:
  cmd.run:
    - name: echo sls_dir={{sls_dir}}
    - cwd: /
""",
            sls="path.to.sls",
        )
        self.assertEqual(
            result["test"]["cmd.run"][0]["name"],
            "echo sls_dir=path{0}to".format(os.sep),
        )

    def test_states_declared_with_shorthand_no_args(self):
        result = self._render_sls(
            """
test:
  cmd.run:
    - name: echo testing
    - cwd: /
test1:
  pkg.installed
test2:
  user.present
"""
        )
        self.assertEqual(len(result), 3)
        for args in (result["test1"]["pkg.installed"], result["test2"]["user.present"]):
            self.assertTrue(isinstance(args, list))
            self.assertEqual(len(args), 0)
        self.assertEqual(result["test"]["cmd.run"][0]["name"], "echo testing")

    def test_adding_state_name_arg_for_dot_state_id(self):
        result = self._render_sls(
            """
.test:
  pkg.installed:
    - cwd: /
.test2:
  pkg.installed:
    - name: vim
""",
            sls="test",
        )
        self.assertEqual(result["test::test"]["pkg.installed"][0]["name"], "test")
        self.assertEqual(result["test::test2"]["pkg.installed"][0]["name"], "vim")

    def test_state_prefix(self):
        result = self._render_sls(
            """
.test:
  cmd.run:
    - name: echo renamed
    - cwd: /

state_id:
  cmd:
    - run
    - name: echo not renamed
    - cwd: /
""",
            sls="test",
        )
        self.assertEqual(len(result), 2)
        self.assertTrue("test::test" in result)
        self.assertTrue("state_id" in result)

    def test_dot_state_id_in_requisites(self):
        for req in REQUISITES:
            result = self._render_sls(
                """
.test:
  cmd.run:
    - name: echo renamed
    - cwd: /

state_id:
  cmd.run:
    - name: echo not renamed
    - cwd: /
    - {0}:
      - cmd: .test

    """.format(
                    req
                ),
                sls="test",
            )
            self.assertEqual(len(result), 2)
            self.assertTrue("test::test" in result)
            self.assertTrue("state_id" in result)
            self.assertEqual(
                result["state_id"]["cmd.run"][2][req][0]["cmd"], "test::test"
            )

    def test_relative_include_with_requisites(self):
        for req in REQUISITES:
            result = self._render_sls(
                """
include:
  - some.helper
  - .utils

state_id:
  cmd.run:
    - name: echo test
    - cwd: /
    - {0}:
      - cmd: .utils::some_state
""".format(
                    req
                ),
                sls="test.work",
            )
            self.assertEqual(result["include"][1], {"base": "test.utils"})
            self.assertEqual(
                result["state_id"]["cmd.run"][2][req][0]["cmd"],
                "test.utils::some_state",
            )

    def test_relative_include_and_extend(self):
        result = self._render_sls(
            """
include:
  - some.helper
  - .utils

extend:
  .utils::some_state:
    cmd.run:
      - name: echo overridden
    """,
            sls="test.work",
        )
        self.assertTrue("test.utils::some_state" in result["extend"])

    def test_multilevel_relative_include_with_requisites(self):
        for req in REQUISITES:
            result = self._render_sls(
                """
include:
  - .shared
  - ..utils
  - ...helper

state_id:
  cmd.run:
    - name: echo test
    - cwd: /
    - {0}:
      - cmd: ..utils::some_state
""".format(
                    req
                ),
                sls="test.nested.work",
            )
            self.assertEqual(result["include"][0], {"base": "test.nested.shared"})
            self.assertEqual(result["include"][1], {"base": "test.utils"})
            self.assertEqual(result["include"][2], {"base": "helper"})
            self.assertEqual(
                result["state_id"]["cmd.run"][2][req][0]["cmd"],
                "test.utils::some_state",
            )

    def test_multilevel_relative_include_beyond_top_level(self):
        self.assertRaises(
            SaltRenderError,
            self._render_sls,
            """
include:
  - ...shared
""",
            sls="test.work",
        )

    def test_start_state_generation(self):
        result = self._render_sls(
            """
A:
  cmd.run:
    - name: echo hello
    - cwd: /
B:
  cmd.run:
    - name: echo world
    - cwd: /
""",
            sls="test",
            argline="-so yaml . jinja",
        )
        self.assertEqual(len(result), 4)
        self.assertEqual(
            result["test::start"]["stateconf.set"][0]["require_in"][0]["cmd"], "A"
        )

    def test_goal_state_generation(self):
        result = self._render_sls(
            """
{% for sid in "ABCDE": %}
{{sid}}:
  cmd.run:
    - name: echo this is {{sid}}
    - cwd: /
{% endfor %}

""",
            sls="test.goalstate",
            argline="yaml . jinja",
        )
        self.assertEqual(len(result), len("ABCDE") + 1)

        reqs = result["test.goalstate::goal"]["stateconf.set"][0]["require"]
        self.assertEqual(set([next(six.itervalues(i)) for i in reqs]), set("ABCDE"))

    def test_implicit_require_with_goal_state(self):
        result = self._render_sls(
            """
{% for sid in "ABCDE": %}
{{sid}}:
  cmd.run:
    - name: echo this is {{sid}}
    - cwd: /
{% endfor %}

F:
  cmd.run:
    - name: echo this is F
    - cwd: /
    - require:
      - cmd: A
      - cmd: B

G:
  cmd.run:
    - name: echo this is G
    - cwd: /
    - require:
      - cmd: D
      - cmd: F
""",
            sls="test",
            argline="-o yaml . jinja",
        )

        sids = "ABCDEFG"[::-1]
        for i, sid in enumerate(sids):
            if i < len(sids) - 1:
                self.assertEqual(
                    result[sid]["cmd.run"][2]["require"][0]["cmd"], sids[i + 1]
                )

        F_args = result["F"]["cmd.run"]
        self.assertEqual(len(F_args), 3)
        F_req = F_args[2]["require"]
        self.assertEqual(len(F_req), 3)
        self.assertEqual(F_req[1]["cmd"], "A")
        self.assertEqual(F_req[2]["cmd"], "B")

        G_args = result["G"]["cmd.run"]
        self.assertEqual(len(G_args), 3)
        G_req = G_args[2]["require"]
        self.assertEqual(len(G_req), 3)
        self.assertEqual(G_req[1]["cmd"], "D")
        self.assertEqual(G_req[2]["cmd"], "F")

        goal_args = result["test::goal"]["stateconf.set"]
        self.assertEqual(len(goal_args), 1)
        self.assertEqual(
            [next(six.itervalues(i)) for i in goal_args[0]["require"]], list("ABCDEFG")
        )

    def test_slsdir(self):
        result = self._render_sls(
            """
formula/woot.sls:
  cmd.run:
    - name: echo {{ slspath }}
    - cwd: /
""",
            sls="formula.woot",
            argline="yaml . jinja",
        )

        r = result["formula/woot.sls"]["cmd.run"][0]["name"]
        self.assertEqual(r, "echo formula/woot")



# coding: utf-8

# Import Python libs
from __future__ import absolute_import

import os
import shutil
import tempfile

import salt.config
import salt.spm
import salt.utils.files
from tests.support.helpers import destructiveTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
from tests.support.unit import TestCase

_F1 = {
    "definition": {
        "name": "formula1",
        "version": "1.2",
        "release": "2",
        "summary": "test",
        "description": "testing, nothing to see here",
    }
}

_F1["contents"] = (
    (
        "FORMULA",
        (
            "name: {name}\n"
            "version: {version}\n"
            "release: {release}\n"
            "summary: {summary}\n"
            "description: {description}"
        ).format(**_F1["definition"]),
    ),
    ("modules/mod1.py", "# mod1.py"),
    ("modules/mod2.py", "# mod2.py"),
    ("states/state1.sls", "# state1.sls"),
    ("states/state2.sls", "# state2.sls"),
)


@destructiveTest
class SPMTestUserInterface(salt.spm.SPMUserInterface):
    """
    Unit test user interface to SPMClient
    """

    def __init__(self):
        self._status = []
        self._confirm = []
        self._error = []

    def status(self, msg):
        self._status.append(msg)

    def confirm(self, action):
        self._confirm.append(action)

    def error(self, msg):
        self._error.append(msg)


class SPMTest(TestCase, AdaptedConfigurationTestCaseMixin):
    def setUp(self):
        self._tmp_spm = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self._tmp_spm, ignore_errors=True)

        minion_config = self.get_temp_config(
            "minion",
            **{
                "spm_logfile": os.path.join(self._tmp_spm, "log"),
                "spm_repos_config": os.path.join(self._tmp_spm, "etc", "spm.repos"),
                "spm_cache_dir": os.path.join(self._tmp_spm, "cache"),
                "spm_build_dir": os.path.join(self._tmp_spm, "build"),
                "spm_build_exclude": [".git"],
                "spm_db_provider": "sqlite3",
                "spm_files_provider": "local",
                "spm_db": os.path.join(self._tmp_spm, "packages.db"),
                "extension_modules": os.path.join(self._tmp_spm, "modules"),
                "file_roots": {"base": [self._tmp_spm]},
                "formula_path": os.path.join(self._tmp_spm, "spm"),
                "pillar_path": os.path.join(self._tmp_spm, "pillar"),
                "reactor_path": os.path.join(self._tmp_spm, "reactor"),
                "assume_yes": True,
                "force": False,
                "verbose": False,
                "cache": "localfs",
                "cachedir": os.path.join(self._tmp_spm, "cache"),
                "spm_repo_dups": "ignore",
                "spm_share_dir": os.path.join(self._tmp_spm, "share"),
            }
        )
        self.ui = SPMTestUserInterface()
        self.client = salt.spm.SPMClient(self.ui, minion_config)
        self.minion_config = minion_config
        for attr in ("client", "ui", "_tmp_spm", "minion_config"):
            self.addCleanup(delattr, self, attr)

    def _create_formula_files(self, formula):
        fdir = os.path.join(self._tmp_spm, formula["definition"]["name"])
        shutil.rmtree(fdir, ignore_errors=True)
        os.mkdir(fdir)
        for path, contents in formula["contents"]:
            path = os.path.join(fdir, path)
            dirname, _ = os.path.split(path)
            if not os.path.exists(dirname):
                os.makedirs(dirname)
            with salt.utils.files.fopen(path, "w") as f:
                f.write(contents)
        return fdir

    def test_build_install(self):
        # Build package
        fdir = self._create_formula_files(_F1)
        with patch("salt.client.Caller", MagicMock(return_value=self.minion_opts)):
            with patch(
                "salt.client.get_local_client",
                MagicMock(return_value=self.minion_opts["conf_file"]),
            ):
                self.client.run(["build", fdir])
        pkgpath = self.ui._status[-1].split()[-1]
        assert os.path.exists(pkgpath)
        # Install package
        with patch("salt.client.Caller", MagicMock(return_value=self.minion_opts)):
            with patch(
                "salt.client.get_local_client",
                MagicMock(return_value=self.minion_opts["conf_file"]),
            ):
                self.client.run(["local", "install", pkgpath])
        # Check filesystem
        for path, contents in _F1["contents"]:
            path = os.path.join(
                self.minion_config["file_roots"]["base"][0],
                _F1["definition"]["name"],
                path,
            )
            assert os.path.exists(path)
            with salt.utils.files.fopen(path, "r") as rfh:
                assert rfh.read() == contents
        # Check database
        with patch("salt.client.Caller", MagicMock(return_value=self.minion_opts)):
            with patch(
                "salt.client.get_local_client",
                MagicMock(return_value=self.minion_opts["conf_file"]),
            ):
                self.client.run(["info", _F1["definition"]["name"]])
        lines = self.ui._status[-1].split("\n")
        for key, line in (
            ("name", "Name: {0}"),
            ("version", "Version: {0}"),
            ("release", "Release: {0}"),
            ("summary", "Summary: {0}"),
        ):
            assert line.format(_F1["definition"][key]) in lines
        # Reinstall with force=False, should fail
        self.ui._error = []
        with patch("salt.client.Caller", MagicMock(return_value=self.minion_opts)):
            with patch(
                "salt.client.get_local_client",
                MagicMock(return_value=self.minion_opts["conf_file"]),
            ):
                self.client.run(["local", "install", pkgpath])
        assert len(self.ui._error) > 0
        # Reinstall with force=True, should succeed
        with patch.dict(self.minion_config, {"force": True}):
            self.ui._error = []
            with patch("salt.client.Caller", MagicMock(return_value=self.minion_opts)):
                with patch(
                    "salt.client.get_local_client",
                    MagicMock(return_value=self.minion_opts["conf_file"]),
                ):
                    self.client.run(["local", "install", pkgpath])
            assert len(self.ui._error) == 0

    def test_failure_paths(self):
        fail_args = (
            ["bogus", "command"],
            ["create_repo"],
            ["build"],
            ["build", "/nonexistent/path"],
            ["info"],
            ["info", "not_installed"],
            ["files"],
            ["files", "not_installed"],
            ["install"],
            ["install", "nonexistent.spm"],
            ["remove"],
            ["remove", "not_installed"],
            ["local", "bogus", "command"],
            ["local", "info"],
            ["local", "info", "/nonexistent/path/junk.spm"],
            ["local", "files"],
            ["local", "files", "/nonexistent/path/junk.spm"],
            ["local", "install"],
            ["local", "install", "/nonexistent/path/junk.spm"],
            ["local", "list"],
            ["local", "list", "/nonexistent/path/junk.spm"],
            # XXX install failure due to missing deps
            # XXX install failure due to missing field
        )

        for args in fail_args:
            self.ui._error = []
            with patch("salt.client.Caller", MagicMock(return_value=self.minion_opts)):
                with patch(
                    "salt.client.get_local_client",
                    MagicMock(return_value=self.minion_opts["conf_file"]),
                ):
                    self.client.run(args)
            assert len(self.ui._error) > 0



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import collections

# Import Salt Libs
import salt.roster.sshconfig as sshconfig
from tests.support import mixins

# Import Salt Testing Libs
from tests.support.mock import mock_open, patch
from tests.support.unit import TestCase

_SAMPLE_SSH_CONFIG = """
Host *
    User user.mcuserface

Host abc*
    IdentityFile ~/.ssh/id_rsa_abc

Host def*
    IdentityFile ~/.ssh/id_rsa_def

Host abc.asdfgfdhgjkl.com
    HostName 123.123.123.123

Host abc123.asdfgfdhgjkl.com
    HostName 123.123.123.124

Host def.asdfgfdhgjkl.com
    HostName 234.234.234.234
"""

_TARGET_ABC = collections.OrderedDict(
    [
        ("user", "user.mcuserface"),
        ("priv", "~/.ssh/id_rsa_abc"),
        ("host", "abc.asdfgfdhgjkl.com"),
    ]
)

_TARGET_ABC123 = collections.OrderedDict(
    [
        ("user", "user.mcuserface"),
        ("priv", "~/.ssh/id_rsa_abc"),
        ("host", "abc123.asdfgfdhgjkl.com"),
    ]
)

_TARGET_DEF = collections.OrderedDict(
    [
        ("user", "user.mcuserface"),
        ("priv", "~/.ssh/id_rsa_def"),
        ("host", "def.asdfgfdhgjkl.com"),
    ]
)

_ALL = {
    "abc.asdfgfdhgjkl.com": _TARGET_ABC,
    "abc123.asdfgfdhgjkl.com": _TARGET_ABC123,
    "def.asdfgfdhgjkl.com": _TARGET_DEF,
}

_ABC_GLOB = {
    "abc.asdfgfdhgjkl.com": _TARGET_ABC,
    "abc123.asdfgfdhgjkl.com": _TARGET_ABC123,
}


class SSHConfigRosterTestCase(TestCase, mixins.LoaderModuleMockMixin):
    def setUp(self):
        self.mock_fp = mock_open(read_data=_SAMPLE_SSH_CONFIG)

    def setup_loader_modules(self):
        return {sshconfig: {}}

    def test_all(self):
        with patch("salt.utils.files.fopen", self.mock_fp):
            with patch("salt.roster.sshconfig._get_ssh_config_file"):
                targets = sshconfig.targets("*")
        self.assertEqual(targets, _ALL)

    def test_abc_glob(self):
        with patch("salt.utils.files.fopen", self.mock_fp):
            with patch("salt.roster.sshconfig._get_ssh_config_file"):
                targets = sshconfig.targets("abc*")
        self.assertEqual(targets, _ABC_GLOB)



# coding: utf-8

# Python libs
from __future__ import absolute_import

import datetime
import logging

# Salt libs
import salt.beacons.wtmp as wtmp
from salt.ext import six
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch

# Salt testing libs
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error
try:
    import dateutil.parser as dateutil_parser  # pylint: disable=unused-import

    _TIME_SUPPORTED = True
except ImportError:
    _TIME_SUPPORTED = False

raw = b"\x07\x00\x00\x00H\x18\x00\x00pts/14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00s/14gareth\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00::1\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x13I\xc5YZf\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
pack = (
    7,
    6216,
    b"pts/14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
    b"s/14",
    b"gareth\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
    b"::1\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
    0,
    0,
    0,
    1506101523,
    353882,
    0,
    0,
    0,
    16777216,
)

log = logging.getLogger(__name__)


class WTMPBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.[s]
    """

    def setup_loader_modules(self):
        return {wtmp: {"__context__": {"wtmp.loc": 2}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}
        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, "Configuration for wtmp beacon must be a list."))

    def test_empty_config(self):
        config = [{}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

    def test_no_match(self):
        config = [
            {
                "users": {
                    "gareth": {
                        "time_range": {
                            "end": "09-22-2017 5pm",
                            "start": "09-22-2017 3pm",
                        }
                    }
                }
            }
        ]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

        with patch("salt.utils.files.fopen", mock_open(b"")) as m_open:
            ret = wtmp.beacon(config)
            call_args = next(six.itervalues(m_open.filehandles))[0].call.args
            assert call_args == (wtmp.WTMP, "rb"), call_args
            assert ret == [], ret

    def test_invalid_users(self):
        config = [{"users": ["gareth"]}]

        ret = wtmp.validate(config)

        self.assertEqual(
            ret, (False, "User configuration for wtmp beacon must be a dictionary.")
        )

    def test_invalid_groups(self):
        config = [{"groups": ["docker"]}]

        ret = wtmp.validate(config)

        self.assertEqual(
            ret, (False, "Group configuration for wtmp beacon must be a dictionary.")
        )

    def test_default_invalid_time_range(self):
        config = [{"defaults": {"time_range": {"start": "3pm"}}}]

        ret = wtmp.validate(config)

        self.assertEqual(
            ret,
            (
                False,
                "The time_range parameter for wtmp beacon must contain start & end options.",
            ),
        )

    def test_users_invalid_time_range(self):
        config = [{"users": {"gareth": {"time_range": {"start": "3pm"}}}}]

        ret = wtmp.validate(config)

        self.assertEqual(
            ret,
            (
                False,
                "The time_range parameter for wtmp beacon must contain start & end options.",
            ),
        )

    def test_groups_invalid_time_range(self):
        config = [{"groups": {"docker": {"time_range": {"start": "3pm"}}}}]

        ret = wtmp.validate(config)

        self.assertEqual(
            ret,
            (
                False,
                "The time_range parameter for wtmp beacon must contain start & end options.",
            ),
        )

    def test_match(self):
        with patch("salt.utils.files.fopen", mock_open(read_data=raw)):
            with patch("struct.unpack", MagicMock(return_value=pack)):
                config = [{"users": {"gareth": {}}}]

                ret = wtmp.validate(config)

                self.assertEqual(ret, (True, "Valid beacon configuration"))

                _expected = [
                    {
                        "PID": 6216,
                        "action": "login",
                        "line": "pts/14",
                        "session": 0,
                        "time": 0,
                        "exit_status": 0,
                        "inittab": "s/14",
                        "type": 7,
                        "addr": 1506101523,
                        "hostname": "::1",
                        "user": "gareth",
                    }
                ]

                ret = wtmp.beacon(config)
                log.debug("{}".format(ret))
                self.assertEqual(ret, _expected)

    @skipIf(not _TIME_SUPPORTED, "dateutil.parser is missing.")
    def test_match_time(self):
        with patch("salt.utils.files.fopen", mock_open(read_data=raw)):
            mock_now = datetime.datetime(2017, 9, 22, 16, 0, 0, 0)
            with patch("datetime.datetime", MagicMock()), patch(
                "datetime.datetime.now", MagicMock(return_value=mock_now)
            ):
                with patch("struct.unpack", MagicMock(return_value=pack)):
                    config = [
                        {
                            "users": {
                                "gareth": {
                                    "time": {
                                        "end": "09-22-2017 5pm",
                                        "start": "09-22-2017 3pm",
                                    }
                                }
                            }
                        }
                    ]

                    ret = wtmp.validate(config)

                    self.assertEqual(ret, (True, "Valid beacon configuration"))

                    _expected = [
                        {
                            "PID": 6216,
                            "action": "login",
                            "line": "pts/14",
                            "session": 0,
                            "time": 0,
                            "exit_status": 0,
                            "inittab": "s/14",
                            "type": 7,
                            "addr": 1506101523,
                            "hostname": "::1",
                            "user": "gareth",
                        }
                    ]

                    ret = wtmp.beacon(config)
                    self.assertEqual(ret, _expected)

    def test_match_group(self):

        for groupadd in (
            "salt.modules.aix_group",
            "salt.modules.mac_group",
            "salt.modules.pw_group",
            "salt.modules.solaris_group",
            "salt.modules.win_groupadd",
        ):
            mock_group_info = {
                "passwd": "x",
                "gid": 100,
                "name": "users",
                "members": ["gareth"],
            }

            with patch("salt.utils.files.fopen", mock_open(read_data=raw)):
                with patch("time.time", MagicMock(return_value=1506121200)):
                    with patch("struct.unpack", MagicMock(return_value=pack)):
                        with patch(
                            "{0}.info".format(groupadd),
                            new=MagicMock(return_value=mock_group_info),
                        ):
                            config = [
                                {
                                    "group": {
                                        "users": {
                                            "time": {
                                                "end": "09-22-2017 5pm",
                                                "start": "09-22-2017 3pm",
                                            }
                                        }
                                    }
                                }
                            ]

                            ret = wtmp.validate(config)

                            self.assertEqual(ret, (True, "Valid beacon configuration"))

                            _expected = [
                                {
                                    "PID": 6216,
                                    "action": "login",
                                    "line": "pts/14",
                                    "session": 0,
                                    "time": 0,
                                    "exit_status": 0,
                                    "inittab": "s/14",
                                    "type": 7,
                                    "addr": 1506101523,
                                    "hostname": "::1",
                                    "user": "gareth",
                                }
                            ]

                            ret = wtmp.beacon(config)
                            self.assertEqual(ret, _expected)



# -*- coding: utf-8 -*-
"""
Test wrapper for running all KitchenSalt tests

All directories in 'tests/kitchen/' will be treated as a separate test under
the KitchenTestCase.
"""
from __future__ import absolute_import

import os

import salt.utils.path
import setup
from salt.modules import cmdmod as cmd
from tests.support.unit import TestCase, skipIf

CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))


@skipIf(not salt.utils.path.which("bundle"), "Bundler is not installed")
class KitchenTestCase(TestCase):
    """
    Test kitchen environments
    """

    @classmethod
    def setUpClass(cls):
        """
        setup kitchen tests
        """
        cls.topdir = "/" + os.path.join(*CURRENT_DIR.split("/")[:-2])
        cls.use_vt = int(os.environ.get("TESTS_LOG_LEVEL")) >= 5
        cmd.run("python setup.py sdist", cwd=cls.topdir)
        cmd.run("bundle install", cwd=CURRENT_DIR)
        cls.env = {
            "KITCHEN_YAML": os.path.join(CURRENT_DIR, ".kitchen.yml"),
            "SALT_SDIST_PATH": os.path.join(
                cls.topdir, "dist", "salt-{0}.tar.gz".format(setup.__version__)
            ),
        }

    @classmethod
    def tearDownClass(cls):
        del cls.topdir
        del cls.env

    def tearDown(self):
        cmd.run(
            "bundle exec kitchen destroy all",
            cwd=os.path.join(CURRENT_DIR, "tests", self.testdir),
            env=self.env,
            use_vt=self.use_vt,
        )
        del self.testdir


def func_builder(testdir):
    def func(self):
        self.testdir = testdir
        if "TESTS_XML_OUTPUT_DIR" in os.environ:
            self.env[
                "TESTS_JUNIT_XML_PATH"
            ] = "{0}/kitchen.tests.{1}.$KITCHEN_SUITE.$KITCHEN_PLATFORM.xml".format(
                os.environ.get("TESTS_XML_OUTPUT_DIR"), self.testdir,
            )
        self.assertEqual(
            cmd.retcode(
                "bundle exec kitchen converge -c 999 all",
                cwd=os.path.join(CURRENT_DIR, "tests", self.testdir),
                env=self.env,
                use_vt=self.use_vt,
            ),
            0,
        )
        self.assertEqual(
            cmd.retcode(
                "bundle exec kitchen verify all",
                cwd=os.path.join(CURRENT_DIR, "tests", self.testdir),
                env=self.env,
                use_vt=self.use_vt,
            ),
            0,
        )

    return func


for testdir in os.listdir(os.path.join(CURRENT_DIR, "tests")):
    setattr(KitchenTestCase, "test_kitchen_{0}".format(testdir), func_builder(testdir))
